/*
 * productionMode.cpp
 *
 *  Created on: Apr 05, 2019
 *      Author: wn
 */


#include "defines.h"

#define MQTT_MAX_PACKET_SIZE 256

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>

#define FASTLED_ESP8266_NODEMCU_PIN_ORDER
#include <FastLED.h>

#include "configuration.h"



void callback(char* topic, byte* payload, unsigned int length);




WiFiClientSecure espClient;
PubSubClient client(espClient);



CRGB leds[NUM_OF_LEDs];


void setup_wifi() {
	delay(10);
	WiFi.mode(WIFI_STA);

	// We start by connecting to a WiFi network
#ifdef DEBUG
	Serial.println();
	Serial.print("Connecting to ");
	Serial.println(configBlock.wifiSsid);
#endif

	WiFi.begin(configBlock.wifiSsid, configBlock.wifiKey);

	while (WiFi.status() != WL_CONNECTED) {
		delay(50);
#ifdef DEBUG
		Serial.print(".");
#endif
	}
	Serial.println("!");

#ifdef DEBUG
	Serial.println("");
	Serial.println("WiFi connected");
	Serial.println("IP address: ");
	Serial.println(WiFi.localIP());
#endif
}




void callback(char* topic, byte* payload, unsigned int length) {
  const uint8_t BUFSIZE = 128;
  if ((length + 1) >= BUFSIZE) {  // 1 for terminating NUL
#ifdef DEBUG
	  Serial.println("Received message too long, ignore it");
#endif
  } else {
    char buffer[BUFSIZE];
    memcpy(buffer, payload, length);
    *(buffer + length) = 0;
#ifdef DEBUG
    Serial.print("Received message: ");
    Serial.print(length);
    Serial.print(" ");
    Serial.print(topic);
    Serial.print(" ");
    Serial.println(buffer);
#endif


    if (! strcmp(topic, configBlock.mqttTopicCommand)) {
    }

    if (! strcmp(topic, configBlock.mqttTopicColorAllLed)) {
    }

    if (! strncmp(topic, configBlock.mqttTopicColorOneLed, strlen(configBlock.mqttTopicColorOneLed))) {

    }

    if (! strcmp(topic, configBlock.mqttTopicStatusAllLed)) {
    	uint8_t v = ! strcmp(buffer, "on") ? 255 : 0;

    	for (uint8_t i = 0; i < NUM_OF_LEDs; i++) {
    		leds[i].r = v;
    		leds[i].g = v;
    		leds[i].b = v;
    	}
    	FastLED.show();
    }
  }
}



void reconnect() {
	// Loop until we're reconnected
	while (!client.connected()) {
#ifdef DEBUG
		Serial.print("Attempting MQTT connection...");
#endif
		// Attempt to connect
		//char clientId[128];
		//snprintf(clientId, 127, "esp%s", WiFi.macAddress().c_str());
		if (client.connect(configBlock.mqttClientId, configBlock.mqttUser, configBlock.mqttPass)) {
#ifdef DEBUG
			Serial.println("connected");
#endif
			client.setCallback(callback);

			// Once connected, publish an announcement...
			client.publish(configBlock.mqttDebugTopic, "hello world");
			client.publish(configBlock.mqttDebugTopic, WiFi.localIP().toString().c_str());

			client.subscribe(configBlock.mqttTopicCommand);
			client.subscribe(configBlock.mqttTopicColorAllLed);
			client.subscribe(configBlock.mqttTopicStatusAllLed);
			char buf[66];
			strncpy(buf, configBlock.mqttTopicColorOneLed, sizeof(buf));
			strncat(buf, "/#", sizeof(buf));
			client.subscribe(buf);
		} else {
#ifdef DEBUG
			Serial.print("failed, rc=");
			Serial.print(client.state());
			Serial.println(" try again in 5 seconds");
#endif
			// Wait 5 seconds before retrying
			delay(5000);
		}
	}
}


bool mqtt_connect() {
	bool reconnected = false;
	if (!client.connected()) {
		reconnect();
		reconnected = true;
	}
	client.loop();
	return reconnected;
}


void setupProduction() {
	setup_wifi();
	client.setServer(configBlock.mqttBroker, configBlock.mqttPort);

    FastLED.addLeds<NEOPIXEL, PIXEL_PIN>(leds, NUM_OF_LEDs);
}


void loopProduction() {
	bool reconnected = mqtt_connect();
	static uint32_t reconnectTime = 0;
	if (reconnected) {
		reconnectTime = millis();
	}



}



/*
#define FASTLED_ESP8266_NODEMCU_PIN_ORDER

#include "RgbLed.h"
#include <FastLED.h>


CRGB leds[2];

void setup() {
    FastLED.addLeds<NEOPIXEL, 1>(leds, 2);

}

void loop() {
	leds[0] = CRGB::Red;
	leds[1] = CRGB::Red;
	FastLED.show();
	delay(500);
	leds[0] = CRGB::Green;
	leds[1] = CRGB::Green;
	FastLED.show();
	delay(500);
	leds[0] = CRGB::Blue;
	leds[1] = CRGB::Blue;
	FastLED.show();
	delay(500);
	leds[0].r = 0;
	leds[0].g = 0;
	leds[0].b = 255;
	leds[1].r = 255;
	leds[1].g = 255;
	leds[1].b = 255;
	FastLED.show();
	delay(500);

	delay(2000);
}
*/



